package com.cultural.heritage.service.good.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.CartRecordMapper;
import com.cultural.heritage.mapper.GoodMapper;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cart.CartRecordVO;
import com.cultural.heritage.model.vo.cart.CartGoodVO;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

@Service
@Slf4j
public class CartRecordServiceImpl extends ServiceImpl<CartRecordMapper, CartRecord> implements CartRecordService {


    @Resource
    private GoodMapper goodMapper;


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;

    /**
     * 更新购物车的商品属性
     */
    @Override
    public boolean updateCartGoodAttribute(CartRecordAddRequest cartRecordAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Long goodId = cartRecordAddRequest.getGoodId();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "商品不存在或者已下架");
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.eq("goodId", goodId);
        CartRecord cartRecord = this.baseMapper.selectOne(queryWrapper);
        if (cartRecord != null) {
            // 如果购物车中已存在该商品, 就叠加数量
            cartRecord.setQuantity(cartRecord.getQuantity() + cartRecordAddRequest.getQuantity());
            cartRecord.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(cartRecord.getQuantity())));
            boolean result = this.updateById(cartRecord);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "更新购物车商品数量失败");
        } else {
            // 如果购物车不存在该商品，就向购物车中添加一项
            CartRecord cartGood = new CartRecord();
            BeanUtils.copyProperties(cartRecordAddRequest, cartGood);
            cartGood.setUserId(userId);
            cartGood.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(cartGood.getQuantity())));
            // 校验
            this.validCart(cartGood, false);
            boolean result = this.save(cartGood);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        }
        return true;
    }





    /**
     * 封装用户购物车中的商品信息
     */
    @Override
    public List<CartRecordVO> transformToCartRecordVOList(List<CartRecord> list) {
        return list.stream().map(cartRecord -> {
            // 封装购物车记录VO
            CartRecordVO cartRecordVO = new CartRecordVO();
            BeanUtils.copyProperties(cartRecord, cartRecordVO);

            // 封装商品类GoodVO
            Long goodId = cartRecord.getGoodId();
            QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", goodId);
            Good good = goodMapper.selectOne(queryWrapper);
            CartGoodVO goodVO = new CartGoodVO();
            BeanUtils.copyProperties(good, goodVO);
            cartRecordVO.setCartGoodVO(goodVO);
            return cartRecordVO;
        }).toList();
    }



    /**
     * 校验购物车中的商品信息
     */
    @Override
    public void validCart(CartRecord cartRecord, boolean update) {
        Long userId = cartRecord.getUserId();
        Long goodId = cartRecord.getGoodId();
        Integer quantity = cartRecord.getQuantity();
        BigDecimal subtotal = cartRecord.getSubtotal();
        if (update) {
            if (userId == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR);
            }
        }
        Good good = goodService.getById(goodId);
        if (good == null || good.getIsShelves() == 0) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "不存在该商品或该商品已下架");
        }
        if (ObjectUtils.isEmpty(quantity) || quantity < 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请传递正确的商品数量");
        }
        if (ObjectUtils.isEmpty(subtotal) || subtotal.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
    }


    /**
     * 校验购物车商品项的准确性
     */
    @Override
    public boolean validIsConsistent(CartRecord cartRecord) {
        // 校验购物车中的商品是否存在
        Long goodId = cartRecord.getGoodId();
        Good good = goodService.getById(goodId);
        if (good == null || good.getIsShelves() == 0) {
            log.info("商品不存在或者已下架, 购物车id为" + cartRecord.getId());
            return false;
        }
        // 校验购物车中的商品数量
        Integer inventory = good.getInventory();
        Integer quantity = cartRecord.getQuantity();
        if (quantity > inventory) {
            log.info("商品库存量不足, 购物车id为" + cartRecord.getId());
            return false;
        }
        return true;
    }



    /**
     * 找出存在问题的购物车id列表
     */
    @Override
    public List<Long> getInvalidCartIds(List<CartRecord> cartRecords) {
        List<CartRecord> cartRecordList = cartRecords.stream().filter(cartRecord -> !validIsConsistent(cartRecord)).toList();
        return cartRecordList.stream().map(CartRecord::getId).toList();
    }
}
